Package: clinDR
Version: 2.5.2
Date: 2025-09-30
Title: Simulation and Analysis Tools for Clinical Dose Response
        Modeling
Authors@R: c(person("Neal","Thomas",role=c("aut","cre"),
	email="snthomas99@gmail.com",comment=c(ORCID = '0000-0002-1915-8487')),
	person("Jing","Wu",role="aut",
	email="Jing_Wu@uri.edu"),
	person("Mike K.","Smith",role="aut",
	email="mike.k.smith@pfizer.com"))
Maintainer: Neal Thomas <snthomas99@gmail.com>
Description: Bayesian and ML Emax model fitting, graphics and simulation for clinical dose
    response.  The summary data from the dose response meta-analyses in  
    Thomas, Sweeney, and Somayaji (2014) <doi:10.1080/19466315.2014.924876> and
    Thomas and Roy (2016) <doi:10.1080/19466315.2016.1256229> 
    Wu, Banerjee, Jin, Menon, Martin, and Heatherington(2017) <doi:10.1177/0962280216684528> 
    are included 
    in the package.  The prior distributions for the Bayesian analyses default to
    the posterior predictive distributions derived from these references.
Depends: R (>= 3.5.0), rstan (>= 2.17.3), shiny
Imports: foreach,graphics,DoseFinding,stats,mvtnorm,utils,
        parallel,doParallel, ggplot2, tidyr, purrr, tibble, dplyr,
        glue, waiter
License: GPL (>= 2)
NeedsCompilation: no
Author: Neal Thomas [aut, cre] (ORCID: <https://orcid.org/0000-0002-1915-8487>),
  Jing Wu [aut],
  Mike K. Smith [aut]
Packaged: 2025-10-02 02:11:52 UTC; snthomas99
Repository: CRAN
Date/Publication: 2025-10-02 19:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 17:06:58 UTC; unix
