export(emaxsim,emaxsimB,plot.emaxsim,plot.emaxsimB,plot.emaxsimobj,
			 plot.emaxsimBobj,print.emaxsim,print.emaxsimB,print.emaxsimobj,
			 print.emaxsimBobj,print.emaxPrior,
			 coef.emaxsimB,coef.emaxsim,sigma.emaxsimB,sigma.emaxsim,vcov.emaxsim,
			 predict.emaxsim,predict.emaxsimB,predict.emaxsimobj,predict.emaxsimBobj,
			 summary.emaxsim,summary.emaxsimB,summary.emaxsimobj,summary.emaxsimBobj,
			 summary.fitEmax,summary.fitEmaxB,
			 update.emaxsimobj,emaxfun,FixedMean,RandEmax,randomEmax,selEstan,
			 showStanModels,targetD,targetCI,predict.emaxalt,SeEmax,emaxalt,startEmax,
			 nllogis,plotB,plot.plotB,plotD, DRDensityPlot,plotBdensity,
			 fitEmax,predict.fitEmax,plot.fitEmax,print.fitEmax,
			 coef.fitEmax,sigma.fitEmax,vcov.fitEmax,
			 fitEmaxB,predict.fitEmaxB,plot.fitEmaxB,print.fitEmaxB,
			 coef.fitEmaxB,sigma.fitEmaxB,
			 emaxPrior.control,prior.control,mcmc.control,compileStanModels,
			 bpchkMonoEmax,checkMonoEmax,targetBeta,solveDose,solveEmax,runSimulations)
import(foreach,shiny)
importFrom("rstan","stan_model","sampling","get_sampler_params", 
					 "rstan_options","stan_version")
importFrom("waiter","spin_fading_circles","use_waiter",
					 "Waiter","waiter_show","waiter_hide")
importFrom("graphics", "abline", "plot", "points","mtext")
importFrom("stats", "as.formula", "binomial", "coef", "sigma",  "glm",
             "glm.control", "lm", "median", "na.omit", "nlm", "nls",
             "nls.control", "optim", "pbeta", "pchisq", "pf", "plogis",
             "pnorm", "ppoints", "predict", "qlogis", "qnorm",
             "quantile", "rbeta", "rbinom", "rnorm", 
						"rt", "runif", "var", "vcov",
             "weighted.mean","rchisq","fitted")
importFrom("mvtnorm","rmvt","rmvnorm")
importFrom("utils","capture.output","flush.console")
importFrom("DoseFinding","Mods","optContr","MCTtest")
importFrom("rstan","sampling","stan_model")
importFrom("parallel","nextRNGStream","detectCores","makeCluster","stopCluster")
importFrom("doParallel","registerDoParallel")
importFrom("ggplot2","ggplot","aes","coord_cartesian","geom_point",
					 "scale_color_manual","scale_shape_manual",
					 "geom_hline","facet_wrap","geom_errorbar",
					 "geom_ribbon","theme_bw","scale_fill_grey","labs",
					 "theme","element_blank","element_line","element_text",
					 "geom_line","xlab","ylab","scale_x_continuous")
importFrom("purrr", "map2","map_dbl")
importFrom("tibble", "as_tibble")
importFrom("dplyr", "mutate", "select", "rename", "bind_rows")
importFrom("tidyr", "pivot_longer")
importFrom("glue", "glue")
S3method("[",emaxsim)
S3method("[",emaxsimB)
S3method(coef,emaxsimB)
S3method(coef,emaxsim)
S3method(coef,fitEmax)
S3method(coef,fitEmaxB)
S3method(sigma,emaxsim)
S3method(sigma,emaxsimB)
S3method(sigma,fitEmax)
S3method(sigma,fitEmaxB)
S3method(vcov,emaxsim)
S3method(vcov,fitEmax)
S3method(plot,emaxsim)
S3method(plot,emaxsimB)
S3method(plot,emaxsimobj)
S3method(plot,emaxsimBobj)
S3method(predict,emaxsim)
S3method(predict,emaxsimB)
S3method(predict,emaxsimobj)
S3method(predict,emaxsimBobj)
S3method(print,emaxsim)
S3method(print,emaxsimB)
S3method(print,emaxsimobj)
S3method(print,emaxsimBobj)
S3method(print,emaxPrior)
S3method(summary,emaxsim)
S3method(summary,emaxsimB)
S3method(summary,emaxsimobj)
S3method(summary,emaxsimBobj)
S3method(summary,fitEmax)
S3method(summary,fitEmaxB)
S3method(update,emaxsimobj)
S3method(predict,emaxalt)
S3method(print,fitEmax)
S3method(print,fitEmaxB)
S3method(predict,fitEmax)
S3method(predict,fitEmaxB)
S3method(plot,fitEmax)
S3method(plot,fitEmaxB)
S3method(plot,plotB)
